#include <stdio.h>
#include <stdlib.h>

#define TAILLE_MAX 5

/* Exercice de colles d'informatique */

/* Titre : Calcul d'un tri topologique, d'un tri préfixe */

/* On appelle DAG un graphe orienté acyclique. Dans cet exercice les
 * graphes sont implémentés avec des sommets qui sont des entiers d'un
 * intervalle d'entiers [|0, n-1|]. On supposera que tous les sommets du
 * graphe sont accessibles depuis l'état d'indice 0.
 *
 * 0 ---> 1 ---> 2 ---> 3
*  |      |      ^
*  |      v      |
*  +----> 4 ---> 5 ---> 6
*  */

/* Dans cet exercice on manipule des listes d'entiers positfs dont le
 * nombre d'éléments ne dépassera jamais la valeur TAILLE_MAX-1, aussi on
 * les représente par un tableau de taille TAILLE_MAX dont les éléments
 * sont stockés dans les premières cases du tableau et suivis de -1. Par
 * exemple la liste [1; 2; 3] sera représentée par le tableau {1, 2, 3,
 * -1, -1} */
typedef int* list;

/* Les graphes sont représentés par table de listes de successeurs */
struct graph_s {
    int   nb_noeuds;            /* le nombre de noeuds */
    list* arcs;                 /* les arcs sortants */
};
typedef struct graph_s* graph;

/* La fonction cree_exemple ci-dessous retourne le graphe exemple
 * ci-dessus */
graph cree_exemple() {
    graph res = malloc(sizeof(struct graph_s));
    res->nb_noeuds = 7;
    res->arcs = malloc(sizeof(int*) * res->nb_noeuds);
    for (int i = 0 ; i  < res->nb_noeuds ; i ++) {
        res->arcs[i] = malloc(TAILLE_MAX * sizeof(int));
        for (int j = 0 ; j  < TAILLE_MAX ; j ++) {
            res->arcs[i][j] = -1;
        }
    }
    fflush(stdout);
    res->arcs[0][0] = 1;        /* le premier arc sortant de 0 est 1 */
    res->arcs[0][1] = 4;        /* le second arc sortant de 0 est 4 */

    res->arcs[1][0] = 2;        /* le premier arc sortant de 1 est 2 */
    res->arcs[1][1] = 4;        /* le second arc sortant de 1 est 4 */

    res->arcs[2][0] = 3;        /* le premier arc sortant de 2 est 3 */

    res->arcs[4][0] = 5;        /* le premier arc sortant de 4 est 5 */

    res->arcs[5][0] = 2;        /* le premier arc sortant de 5 est 2 */
    res->arcs[5][1] = 6;        /* le second arc sortant de 5 est 6 */

    return res;
}

/* La fonction cree_mechant_exemple ci-dessous retourne un DAG de taille 2
 * nb+1 */
graph cree_mechant_exemple(int nb) {
    graph res = malloc(sizeof(struct graph_s));
    res->nb_noeuds = 2 * nb + 1;
    res->arcs = malloc(sizeof(int*) * res->nb_noeuds);
    for (int i = 0 ; i  < res->nb_noeuds ; i ++) {
        res->arcs[i] = malloc(TAILLE_MAX * sizeof(int));
        for (int j = 0 ; j  < TAILLE_MAX ; j ++) {
            res->arcs[i][j] = -1;
        }
    }
    res->arcs[0][0] = 1;
    res->arcs[0][1] = 2;
    for (int i = 1 ; i < res->nb_noeuds-2 ; i ++) {
        if (i % 2 == 0) {
            res->arcs[i][0] = i+1;
            res->arcs[i][1] = i+2;
        } else {
            res->arcs[i][0] = i+2;
            res->arcs[i][1] = i+3;
        }
    }
    return res;
}

/* Q1. Écrire une fonction prenant en argument un DAG et affichant les
 * sommets dans l'ordre d'un tri topologique */

/* Q2. Même question mais on veut cette fois que la fonction retourne une
 * liste chaînée (à vous de définir le type liste chaînée) correspondant à
 * un tri topologique */

/* Q3. Dans cette question on ne suppose plus que le graphe est acyclique,
 * écrire une fonction retournant un tri préfixe (stocké dans une liste
 * chaînée) du graphe passé en paramètre. */
